/* Copyright (c) 2000-2001 ActiveState SRL. 
   See the file LICENSE.txt for licensing information. */

/* -*- Mode: Java; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * Komodo's Find and Replace results tab control (a tab in the output pane)
 *
 * TODO:
 *  - try to refind results that have moved from buffer editting since search
 */

var findResultsLog = getLoggingMgr().getLogger('findResultsTab');
//findResultsLog.setLevel(LOG_DEBUG);

var gFindResultsView_Bound = false; // Is the nsITreeView instance is bound to
                                    // the find results <tree>.


//---- internal utility routines

function _OptionDesc(options, pattern)
{
    var optionDesc = "";
    if (options.matchWord) {
        optionDesc += " Match whole word.";
    }
    if (options.caseSensitivity == options.FOC_SENSITIVE
        ||
         (
           options.caseSensitivity == options.FOC_SMART
           &&
           pattern.toLowerCase() != pattern
         )
       )
    {
        optionDesc += " Match case.";
    }
    return optionDesc;
}



//---- interface routines

function FindResultsTab_OnLoad()
{
    // Should be called in the onload handler for the window containing the
    // find results tab XUL.
    gFindResultsView = Components.classes[
        '@activestate.com/koFindResultsView;1']
        .createInstance(Components.interfaces.koIFindResultsView);
    if (!gFindResultsView) {
        log.error("FindResultsTab_OnLoad: couldn't create a koFindResultsView");
        return;
    }
    _FindResultsTab_ClearUI(window);
    gFindResultsView_Bound = false;
}

// Ensure the nsITreeView instance is bound to the <tree>.
function _FindResultsTab_EnsureBound()
{
    if (gFindResultsView_Bound) return;
    var treeWidget = document.getElementById("findresults");
    var boxObject = treeWidget.treeBoxObject
                  .QueryInterface(Components.interfaces.nsITreeBoxObject);
    // we set the flag to true before we call FindResultsTab_Show
    // otherwise we'll end up in infinite recursion.
    gFindResultsView_Bound = true;
    if (boxObject.view == null) {
        // We are in a collapsed state-- we need to force the tree to be visible
        // before we can assign the view to it.
        findResultsLog.info("_FindResultsTab_EnsureBound: forcing it to show");
        FindResultsTab_Show();
    }
    boxObject.view = gFindResultsView;
}

function FindResultsTab_GetView()
{
    findResultsLog.info("FindResultsTab_GetView()");
    _FindResultsTab_EnsureBound();
    return gFindResultsView;
}

function FindResultsTab_Clear()
{
    findResultsLog.info("FindResultsTab_Clear()");
    _FindResultsTab_EnsureBound();
    // Clear the list of find results.
    gFindResultsView.Clear(0);
    _FindResultsTab_ClearUI();
}

function _FindResultsTab_ClearUI()
{
    _FindResultsTab_UpdateSortIndicators(null);
    var descWidget = document.getElementById("findresults-desc");
    descWidget.setAttribute("value", "No current find/replace results.");
    descWidget.removeAttribute("baseDesc");
    document.getElementById('findresults-findnext-button').setAttribute('disabled', 'true')
}


function FindResultsTab_DescribeFind(pattern, context, options, patternAlias)
{
    findResultsLog.info("FindResultsTab_DescribeFind()");
    _FindResultsTab_EnsureBound();
    // Set the tab name appropriately.
    var tabWidget = document.getElementById("findresults_tab");
    tabWidget.setAttribute("label", "Find Results");

    // Add a description of what was found.
    var descWidget = document.getElementById("findresults-desc");
    if (!patternAlias) {
        patternAlias = "occurrences of '" +pattern + "'";
    }
    var baseDesc = "Found all " + patternAlias + " in " + context.name + "."
                   + _OptionDesc(options, pattern);

    descWidget.setAttribute("baseDesc", baseDesc);
    descWidget.setAttribute("value", baseDesc);

    var filenameColWidget = document.getElementById("findresults-filename");
    if (context.type == Components.interfaces.koIFindContext.FCT_CURRENT_DOC
        || context.type == Components.interfaces.koIFindContext.FCT_SELECTION)
    {
        filenameColWidget.setAttribute("hidden", "true");
    } else {
        filenameColWidget.setAttribute("hidden", "false");
    }
    document.getElementById('findresults-findnext-button')
            .removeAttribute('disabled');
}


function FindResultsTab_DescribeReplace(pattern, replacement, context, options)
{
    findResultsLog.info("FindResultsTab_DescribeReplace()");
    _FindResultsTab_EnsureBound();
    // Set the tab name appropriately.
    var tabWidget = document.getElementById("findresults_tab");
    tabWidget.setAttribute("label", "Replace Results");

    // Add a description of what was replaced.
    var descWidget = document.getElementById("findresults-desc");
    var baseDesc = "Replaced all '" + pattern + "' with '" + replacement
                   + "' in " + context.name + "."
                   + _OptionDesc(options, pattern);

    descWidget.setAttribute("baseDesc", baseDesc);
    descWidget.setAttribute("value", baseDesc);

    var filenameColWidget = document.getElementById("findresults-filename");
    if (context.type == Components.interfaces.koIFindContext.FCT_CURRENT_DOC
        || context.type == Components.interfaces.koIFindContext.FCT_SELECTION)
    {
        filenameColWidget.setAttribute("hidden", "true");
    } else {
        filenameColWidget.setAttribute("hidden", "false");
    }
    document.getElementById('findresults-findnext-button')
            .removeAttribute('disabled');
}


function FindResultsTab_AddFindResult(url, startIndex, endIndex, value,
                                      fileName, lineNum, columnNum, context)
{
    findResultsLog.info("FindResultsTab_AddFindResult()");
    _FindResultsTab_EnsureBound();
    //XXX should raise exception if not in find all "mode"
    gFindResultsView.AddFindResult(url, startIndex, endIndex, value, fileName,
                                   lineNum, columnNum, context);
    var descWidget = document.getElementById("findresults-desc");
    var desc = descWidget.getAttribute("baseDesc");
    var numHits = gFindResultsView.rowCount;
    if (numHits == 1) {
        desc += " (1 result found";
    } else {
        desc += " (" + numHits + " results found";
    }
    var numUrls = gFindResultsView.GetNumUrls();
    if (numUrls == 1) {
        desc += " in 1 file)";
    } else {
        desc += " in " + numUrls + " files)";
    }
    descWidget.setAttribute("value", desc);
}


function FindResultsTab_AddReplaceResult(url, startIndex, endIndex, value,
                                         replacement, fileName, lineNum,
                                         columnNum, context)
{
    findResultsLog.info("FindResultsTab_AddReplaceResult()");
    _FindResultsTab_EnsureBound();
    //XXX should raise exception if not in replace all "mode"
    gFindResultsView.AddReplaceResult(url, startIndex, endIndex, value,
                                      replacement, fileName, lineNum,
                                      columnNum, context);
    var descWidget = document.getElementById("findresults-desc");
    var desc = descWidget.getAttribute("baseDesc");
    var numHits = gFindResultsView.rowCount;
    if (numHits == 1) {
        desc += " (1 replacement made";
    } else {
        desc += " (" + numHits + " replacements made";
    }
    var numUrls = gFindResultsView.GetNumUrls();
    if (numUrls == 1) {
        desc += " in 1 file)";
    } else {
        desc += " in " + numUrls + " files)";
    }
    descWidget.setAttribute("value", desc);
}


function FindResultsTab_Show()
{
    findResultsLog.info("FindResultsTab_Show()");
    // If necessary, open output pane in komodo.xul
    uilayout_ensureOutputPaneShown(window);

    // switch to "Find Results" tab
    var tabsWidget = document.getElementById("output_tabs");
    var tabWidget = document.getElementById("findresults_tab");
    tabsWidget.selectedItem = tabWidget; 

    // (controversial?) give the find results tab the focus
    //XXX does not work because the input buffer is passing focus back to the
    //editor
    var findResultsWidget = document.getElementById("findresults-body");
    findResultsWidget.focus()
    // Now the widget can be bound to the view -- make sure that happens
    _FindResultsTab_EnsureBound();
}


function FindResultsTab_OnClick(event)
{
    findResultsLog.info("FindResultsTab_OnClick()");
    _FindResultsTab_EnsureBound();
    // c.f. mozilla/mailnews/base/resources/content/threadPane.js
    var t = event.originalTarget;

    // single-click on a column
    if (t.localName == "treecol") {
        _FindResultsTab_ColumnClick(t.id);
    }

    // double-click in the tree body
    else if (event.detail == 2 && t.localName == "treechildren") {
        _FindResultsTab_DoubleClick();
    }
}


function _FindResultsTab_ColumnClick(columnID)
{
    gFindResultsView.Sort(columnID);
    _FindResultsTab_UpdateSortIndicators(window, columnID);
}


function FindResultsTab_NextResult() {
    findResultsLog.info("FindResultsTab_NextResult()");
    _FindResultsTab_EnsureBound();
    var treeWidget = document.getElementById("findresults");
    if (treeWidget.treeBoxObject.view.rowCount < 1) {
        alert("Could not find next result because the find results "+
              "list is empty.");
        FindResultsTab_Show();
        return;
    }
    var i = treeWidget.currentIndex;
    if (i == -1 || i >= treeWidget.treeBoxObject.view.rowCount-1) {
        i = 0;
    } else {
        i += 1;
    }
    treeWidget.treeBoxObject.view.selection.currentIndex = i;
    treeWidget.treeBoxObject.view.selection.select(i);
    treeWidget.treeBoxObject.ensureRowIsVisible(i);
    _FindResultsTab_DoubleClick();
    
}

function _FindResultsTab_DoubleClick()
{
try {
    // Jump to the find/replace result.

    // Open and/or switch to the appropriate file.
    var treeWidget = document.getElementById("findresults");
    var i = treeWidget.currentIndex;
    //XXX Note that "url" is the current (bad) name for the view ID, which
    //    itself is poorly represented by the display path of the file. Note
    //    that this may not translate to a valid URL if the view is untitled.
    var displayPath = gFindResultsView.GetUrl(i);
    try {
        open_openDisplayPath(displayPath);
    } catch(ex) {
        var msg = "Error opening '"+displayPath+"'.\n";
        alert(msg);
        findResultsLog.error(msg+" "+ex);
        return;
    }
    var view = gViewMgr.currentView;
    var scimoz = view.scintilla.scimoz;
    
    // Try to find the match or replacement result. If it cannot be
    // found then just go to the start of the line where it used to be.
    // XXX Could try and be more sophisticated in the search.
    var start = gFindResultsView.GetStartIndex(i);
    var end = gFindResultsView.GetEndIndex(i);
    var value = gFindResultsView.GetValue(i);
    var replacement = gFindResultsView.GetReplacement(i);
    // - first try the original indeces (if the file has not changed the result
    //   should still be there)
    //   XXX This *can* fluke to the wrong result if the buffer has changed.
    //       The *correct* answer would be to only attempt this if the buffer
    //       is identical to when the find-/replace-all was done.
    if ((replacement && scimoz.getTextRange(start, end) == replacement)
        || scimoz.getTextRange(start, end) == value)
    {
        findResultsLog.info("Jump To Find Result: found at the original indices\n");
        scimoz.setSel(start, end);
    }
    // - next, try using the line and column number to find it again (this will
    //   survive basic changes in the document that do not add or remove lines)
    else {
        var lineNum = gFindResultsView.GetLineNum(i);
        var columnIndex = gFindResultsView.GetColumnNum(i);
        var lineStartIndex = scimoz.positionFromLine(lineNum-1);
        var possibleValue = scimoz.getTextRange(lineStartIndex+columnIndex,
                                                lineStartIndex+columnIndex
                                                +value.length);
        var possibleReplacement = scimoz.getTextRange(lineStartIndex+columnIndex,
                                                      lineStartIndex+columnIndex
                                                      +replacement.length);
        if (replacement && possibleReplacement == replacement) {
            findResultsLog.info("Jump To Find Result: found replacement, '"+replacement+"', by line:col\n");
            scimoz.setSel(lineStartIndex+columnIndex, lineStartIndex+columnIndex+replacement.length);
        } else if (possibleValue == value) {
            findResultsLog.info("Jump To Find Result: found value by line:col\n");
            scimoz.setSel(lineStartIndex+columnIndex, lineStartIndex+columnIndex+value.length);
        }

        // - next, try searching within the current line
        //   XXX Note, this bails if there is more than one possible match on
        //       the current line. This could attempt to be more intelligent,
        //       like perhaps selecting the match that is closest to the
        //       original column number.
        else {
            var lineText = scimoz.getTextRange(lineStartIndex,
                            scimoz.getLineEndPosition(lineNum-1));
            var valueIndex = lineText.indexOf(value);
            var replacementIndex = lineText.indexOf(replacement);
            if (replacement && replacementIndex != -1
                && lineText.lastIndexOf(replacement) == replacementIndex)
            {
                findResultsLog.info("Jump To Find Result: found replacement by searching in the current line\n");
                scimoz.setSel(lineStartIndex+replacementIndex, lineStartIndex+replacementIndex+replacement.length);
            } else if (valueIndex != -1
                       && lineText.lastIndexOf(value) == valueIndex)
            {
                findResultsLog.info("Jump To Find Result: found value by searching in the current line\n");
                scimoz.setSel(lineStartIndex+valueIndex, lineStartIndex+valueIndex+value.length);
            }

            // - XXX next, could try to search a couple lines above and below
            //   but for now will just give up
            else {
                findResultsLog.info("Jump To Find Result: did not find it, just using current line\n");
                //XXX This ends up on the first line if this line is now off
                //    the bottom. This should instead go to the LAST line.
                scimoz.setSel(lineStartIndex, lineStartIndex);

                // Let the user know about the problem on the status bar.
                StatusBar_AddMessage(
                    "The specified text has been moved or deleted.", "find",
                    3000, true);
            }
        }
    }

    // If the invoked find result is the last visible one and there are more,
    // then scroll the list of find results by one.
    // XXX Could also scroll up if double-clicking on top visible item.
    var box = treeWidget.treeBoxObject;
    if (box.getLastVisibleRow() <= i && gFindResultsView.rowCount > i+1) {
        box.scrollByLines(1);
    }
    scimoz.chooseCaretX();

    // transfer focus to the editor
    view.setFocus();
} catch (e) {
    log.exception(e);
}
}


function FindResultsTab_OnKeyPress(event)
{
    findResultsLog.info("FindResultsTab_OnKeyPress()");
    if (event.keyCode == 13) {
        _FindResultsTab_DoubleClick();
        return false;
    }
    return true;
}


function _FindResultsTab_UpdateSortIndicators(sortId)
{
    var sortedColumn = null;

    // set the sort indicator on the column we are sorted by
    if (sortId) {
        sortedColumn = document.getElementById(sortId);
        if (sortedColumn) {
            var sortDirection = sortedColumn.getAttribute("sortDirection");
            if (sortDirection && sortDirection == "ascending") {
                sortedColumn.setAttribute("sortDirection", "descending");
            } else {
                sortedColumn.setAttribute("sortDirection", "ascending");
            }
        }
    }

    // remove the sort indicator from all the columns
    // except the one we are sorted by
    var currCol = document.getElementById("findresults").firstChild.firstChild;
    while (currCol) {
        while (currCol && currCol.localName != "treecol") {
            currCol = currCol.nextSibling;
        }
        if (currCol && (currCol != sortedColumn)) {
            currCol.removeAttribute("sortDirection");
        }
        if (currCol) {
            currCol = currCol.nextSibling;
        }
    }
}


