/* Copyright (c) 2003 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */

var log = getLoggingMgr().getLogger("pref-intl");

// Constants and global variables
var _elementIDs = [
    "encodingEnvironment", "encodingDefault", "encodingAutoDetect",
    "encodingXMLDec", "encodingHTML", "defaultDateFormat"
];

const XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
const tag = 'chrome://jaguar/content/pref/pref-intl.xul';
var timeSvc = Components.classes["@activestate.com/koTime;1"].
              getService(Components.interfaces.koITime);

var koEncodingServices = Components.classes["@activestate.com/koEncodingServices;1"]
            .getService(Components.interfaces.koIEncodingServices);
var langPrefs;
var currentLangPrefs;
var newHasChanged = false;
var startEncoding;
var latestLanguage;
var langToUse;
var dialog = null;

function PrefIntl_OnLoad() {
    log.info("PrefIntl_OnLoad");
    try {
        // set up UI elements
        dialog = {};
        dialog.newlangList = document.getElementById('newlangList');
        dialog.newencodingList = document.getElementById('newencodingList');
        dialog.bomBox = document.getElementById('bomBox');
        dialog.encodingDefault = document.getElementById('encodingDefault');
        dialog.encodingEnvironment = document.getElementById('encodingEnvironment');
        dialog.encodingAutoDetect = document.getElementById('encodingAutoDetect');
        dialog.encodingXMLDec = document.getElementById('encodingXMLDec');
        
        // order is important here
        // 1 - set up oepn encoding list
        // 2 - load the page with preferences, which will set the 'pref' values
        // 3 - do new excoding, which is the manual getting of prefs for per language encoding
        doDefaultEncoding();    

        // register with preferences
        parent.hPrefWindow.onpageload(tag);
        parent.hPrefWindow.registerOKCallbackFunc(PrefIntl_OkCallback, tag);
        doNewEncoding();
    
        if (dialog.encodingEnvironment.checked)
            changeDefaultEncoding();
    } catch(ex) {
        log.error(ex);
    }
}

function OnPreferencePageInitalize(prefset) {
    log.info("OnPreferencePageInitalize");
    try {
        var data = new Object();

        // set up the prefs
        data.langPrefs = prefset.getPref('languages');
        data.langPrefs.parent = prefset;    
        data.view = parent.opener.gViewMgr.currentView;
        data.mainWin = parent.opener;
        data.setEncoding = setEncoding; // needs to be called when this page not loaded.
        data.offPanel = false;
        if (data.view)
            data.langToUse = data.view.document.language;
        else   // default
            data.langToUse = "Text";

        // "Date & Time" stuff.
        data.defaultDateFormat = prefset.getStringPref("defaultDateFormat");
    
        return data;
    } catch(ex) {
        log.error(ex);
    }
}

function PrefIntl_OkCallback(prefset, customData)  {
    log.info("PrefIntl_OkCallback");
    try {
        //if (customData.newHasChanged)  {
            if (!customData.setEncoding(customData))  {  // check if encoding is appropriate
                //XXX This *must* present some UI to the user. How are they to know
                //    what is invalid here?
                return false;
            }
            else  {
                prefset.setPref("languages", customData.langPrefs);
            }
        //}

        // "Date & Time" validation.
        var defaultDateFormat = null;
        var dateFormatWidget = document.getElementById("defaultDateFormat");
        if (dateFormatWidget != null) {
            // If this is the current panel then this OkCallback is called before
            // the OnPreferencePanelUnload so we get the value from the widget.
            defaultDateFormat = dateFormatWidget.value;
        } else {
            // Otherwise (this is NOT the current panel), get the value from the
            // custom data object on which the value was set in
            // OnPreferencePanelUnload.
            defaultDateFormat = customData.defaultDateFormat;
        }
        if (!defaultDateFormat) {
            //XXX Would like to check defaultDateFormat.strip() but JavaScript does
            //    not have this method.
            alert("Internationalization: You must enter some non-empty date format.");
            return false;
        }

        return true;
    } catch (ex) {
        log.error(ex);
    }
}

// Start up & returning to this panel in a session
function OnPreferencePageLoading(prefset, data) {
    log.info("OnPreferencePageLoading");
    try {
        data.newHasChanged = false;
        data.offPanel = false;
        langPrefs = data.langPrefs;
        langPrefs.parent = data.langPrefs.parent;
        langToUse = data.langToUse;
    
        //XXX Shouldn't I be grabbing the date format from "data"?
        _setupDateShortcuts();
        setDateFormatExample();
    } catch (ex) {
        log.error(ex);
    }
}    

// Switches to another prefs panel
function OnPreferencePageUnloading(prefset, data) {
    log.info("OnPreferencePageUnloading");
    var retval = true;
    
    data.newHasChanged = newHasChanged;
    data.langPrefs = langPrefs;
    data.langPrefs.parent = langPrefs.parent;
    data.langToUse = dialog.newlangList.selection;
    if (!data.setEncoding(data))  {  // check if encoding is appropriate
        // stay on this panel
        //var prefPanelTree = parent.document.getElementById( "prefsTree" );
        //prefPanelTree.selectedItem = parent.document.getElementById('fileSettingsItem');
        //parent.hPrefWindow.switchPage();
        retval = false;
    }
    else  {
        data.offPanel = true;
    }

    // "Date & Time" stuff
    var dateFormatWidget = document.getElementById("defaultDateFormat");
    data.defaultDateFormat = dateFormatWidget.value;

    //XXX Even though the current prefs system ignores this retval.
    return retval;
}

function doNewEncoding()  {
    var count = new Object();
    var temp = new Object();
    koEncodingServices.enumerateEncodings(temp,count);
    var encodings = temp.value;

    var init_hierarchy = koEncodingServices.encoding_hierarchy;
    var menupopup = encodingMenu_buildHierarchy(init_hierarchy, true, 'changeNewEncoding(this)');

    // Add default option to the top of the list -> Auto-Detect
    var menuitem = document.createElementNS(XUL_NS, 'menuitem');
    menuitem.setAttribute('label', 'Default Encoding');
    menuitem.setAttribute('data', 'Default Encoding');
    menuitem.setAttribute('oncommand', 'changeNewEncoding(this)');
    menupopup.insertBefore(menuitem, menupopup.childNodes[0]);

    dialog.newencodingList.removeChild(dialog.newencodingList.firstChild);
    dialog.newencodingList.appendChild(menupopup);
    dialog.newencodingList.selectedItem = dialog.newencodingList.firstChild.firstChild;
    changeNewLanguage(null, langToUse);
}

function doBOM(encodingInfo)  {
    var bom = encodingInfo.byte_order_marker != '';
    if (bom)  {
        dialog.bomBox.setAttribute('disabled', 'false');
        return true;
    }
    else  {
        dialog.bomBox.setAttribute('disabled', 'true');
        return false;  
    }
}

function _setSelectedEncoding(elt,encoding) {
    var items = elt.getElementsByTagName("menuitem");
    for (var i=0;i<items.length;i++) {
        var item = items[i];
        //dump("Item value is " + item.getAttribute("value") + ", label is " + item.getAttribute("label") + "\n");
        if (item.getAttribute('data') == encoding) {
            //dump("set menu popup to " + item.getAttribute("label") + "\n");
            elt.setAttribute("value", encoding);
            elt.selectedItem = item;
            return;
        }
    }
}

function changeNewLanguage(item, name)  {
    var encoding;
    if (item == null)  {  // startup - first time in
        latestLanguage = name;
        currentLangPrefs = langPrefs.getPref('languages/'+latestLanguage);  // make sure to qualify the pref set
        currentLangPrefs.parent = langPrefs;
        dialog.newlangList.selection = name;
    }
    else  {
        var value = dialog.newlangList.selection;
        latestLanguage = value;
        currentLangPrefs = langPrefs.getPref('languages/'+latestLanguage);  // make sure to qualify the pref set
    }

    if (currentLangPrefs.hasPref(latestLanguage+'/newEncoding'))  {
        encoding = currentLangPrefs.getStringPref(latestLanguage+'/newEncoding');
    } else  {
        log.error("Could not retrieve the language encoding from preferences.");
    }

    _setSelectedEncoding(dialog.newencodingList,encoding);
    
    if (encoding == 'Default Encoding') {
        dialog.bomBox.setAttribute('checked', false);
        dialog.bomBox.setAttribute('disabled', 'true');
    } else {
        var encodingInfo = koEncodingServices.get_encoding_info(encoding);
        if (doBOM(encodingInfo))  {    
            var bomPref = currentLangPrefs.getBooleanPref(latestLanguage+'/newBOM');
            dialog.bomBox.setAttribute('checked', bomPref);
        }    
    }
}

function changeNewEncoding(item)  {
    dialog.newencodingList.selectedItem = item;
    var newEncoding = item.getAttribute('data');
    currentLangPrefs.setStringPref(latestLanguage+'/newEncoding', newEncoding);
    if (newEncoding == 'Default Encoding') {
        dialog.bomBox.setAttribute('checked', false);
        dialog.bomBox.setAttribute('disabled', 'true');
    } else 
    if (doBOM(koEncodingServices.get_encoding_info(newEncoding)))  {
        var bomPref = currentLangPrefs.getBooleanPref(latestLanguage+'/newBOM');
        dialog.bomBox.setAttribute('checked', bomPref);        
    }
    newHasChanged = true;
}

function changeNewBOM(box)  {
    try {
        if (box.hasAttribute('checked') && box.getAttribute('checked') == 'true') {
            box.removeAttribute('checked');
            currentLangPrefs.setBooleanPref(latestLanguage+'/newBOM', false);
        } else {
            box.setAttribute('checked', 'true');
            currentLangPrefs.setBooleanPref(latestLanguage+'/newBOM', true);
        }
        newHasChanged = true;
    } catch (e) {
        log.exception(e);
    }
}

// Checks to see if encoding selected is appropriate for the file type
function setEncoding(customData)  {
    if (!customData.offPanel)  {
        var koEncoding = Components.classes["@activestate.com/koEncoding;1"]
                    .createInstance(Components.interfaces.koIEncoding);
        var encoding_name = dialog.newencodingList.selectedItem.getAttribute('data');
        if (encoding_name == 'Default Encoding') {
            encoding_name = dialog.encodingDefault.firstChild.firstChild.getAttribute('data');
        }
        koEncoding.python_encoding_name = encoding_name;
        koEncoding.use_byte_order_marker = (dialog.bomBox.getAttribute('disabled') == 'false'
                                        && dialog.bomBox.getAttribute('checked') == 'true')

        var languageRegistry = Components.classes["@activestate.com/koLanguageRegistryService;1"]
                .getService(Components.interfaces.koILanguageRegistryService);
        var language = languageRegistry.getLanguage(dialog.newlangList.selection);
        
        var warning = language.getEncodingWarning(koEncoding);
        if (warning == "" || 
            customData.mainWin.dialog_yesNo(
                warning +
                " Are you sure that you want to change the encoding?",
                "No"
            ) == "Yes"
           ) {
            return true;
        }
        else  {
            // reset the pref ??
            return false;
        }
    }
    else  {
        return true;  
    }
}

function doDefaultEncoding()  {
    var count = new Object();
    var temp = new Object();
    koEncodingServices.enumerateEncodings(temp,count);
    var encodings = temp.value;

    var init_hierarchy = koEncodingServices.encoding_hierarchy;
    var menupopup = encodingMenu_buildHierarchy(init_hierarchy, true, 'changeOpenEncoding(this)');
    dialog.encodingDefault.removeChild(dialog.encodingDefault.firstChild);
    dialog.encodingDefault.appendChild(menupopup);
    dialog.encodingDefault.selectedItem = dialog.encodingDefault.firstChild.firstChild;
}

function _getSelectedEncoding(elt,encoding) {
    var items = elt.getElementsByTagName("menuitem");
    for (var i=0;i<items.length;i++) {
        var item = items[i];
        //dump("Item value is " + item.getAttribute("value") + ", label is " + item.getAttribute("label") + "\n");
        if (item.getAttribute('data') == encoding) {
            //dump("set menu popup to " + item.getAttribute("label") + "\n");
            elt.setAttribute("value", encoding);
            return item;
        }
    }
    return null;
}

function changeDefaultEncoding() {
    var _InitSvc = Components.classes["@activestate.com/koInitService;1"].
                    getService(Components.interfaces.koIInitService);
    var locale = _InitSvc.getStartupLocale();
    var encoding = locale.split('.')[1].toLowerCase();
    if (dialog.encodingEnvironment.checked) {
        // disable the menu
        var selectedItem = _getSelectedEncoding(dialog.encodingDefault,encoding);
        dialog.encodingDefault.removeChild(dialog.encodingDefault.firstChild);
        var menupopup = document.createElementNS(XUL_NS, 'menupopup');
        menupopup.appendChild(selectedItem);
        dialog.encodingDefault.appendChild(menupopup);
    } else {
        // enable the menu
        doDefaultEncoding();
    }
    _setSelectedEncoding(dialog.encodingDefault,encoding);
}

function changeOpenEncoding(item)  {
    dialog.encodingDefault.selectedItem = item;
}


//---- "Date & Time" groupbox methods

function setDateFormatExample()
{
    log.info("setDateFormatExample");
    try {
        var secsNow = timeSvc.time();
        var timeTupleNow = timeSvc.localtime(secsNow, new Object());
        var format = document.getElementById("defaultDateFormat").value
        var example = timeSvc.strftime(format, timeTupleNow.length, timeTupleNow);
        document.getElementById("dateFormatExample").value = example;
    } catch(ex) {
        log.error("setDateFormatExample error: "+ex);
    }
}


function _insertShortcut(textbox, shortcut, replaceAll /* =false*/)
{
    if (typeof replaceAll == 'undefined') replaceAll = false;
    if (replaceAll) {
        textbox.value = shortcut;
        textbox.selectionStart = 0;
        textbox.selectionEnd = shortcut.length;
    } else {
        var oldValue = textbox.value;
        var selStart = textbox.selectionStart;
        var selEnd = textbox.selectionEnd;
        
        var newValue = oldValue.slice(0, selStart) + shortcut
                       + oldValue.slice(selEnd, oldValue.length);
        textbox.value = newValue;
        textbox.selectionStart = selStart + shortcut.length;
        textbox.selectionEnd = selStart + shortcut.length;
    }
}


// Get the shortcut string from the menuitem widget and append it to
// the current command string.
function insertDateShortcut(shortcutWidget, replaceAll /* =false */)
{
    if (typeof replaceAll == 'undefined') replaceAll = false;
    var shortcut = shortcutWidget.getAttribute("shortcut");
    var textbox = document.getElementById("defaultDateFormat");
    _insertShortcut(textbox, shortcut, replaceAll);
    setDateFormatExample();
}

// Set the labels for menuitems on the date shortcuts menupopup that don't have
// a hardcoded label.
function _setupDateShortcuts()
{
    log.info("_setupDateShortcuts");
    var menupopup = document.getElementById("dateShortcutsPopup");
    var menuitems = menupopup.getElementsByTagName("menuitem");
    var secsNow = timeSvc.time();
    var timeTupleNow = timeSvc.localtime(secsNow, new Object());
    var i, item, shortcut, label;
    for (i = 0; i < menuitems.length; ++i) {
        item = menuitems[i];
        if (!item.hasAttribute("label")) {
            shortcut = item.getAttribute("shortcut");
            label = timeSvc.strftime(shortcut, timeTupleNow.length,
                                     timeTupleNow);
            item.setAttribute("label", label);
        }
    }
}

