/* Copyright (c) 2000-2003 ActiveState Corporation.
   See the file LICENSE.txt for licensing information. */

/* Komodo's New File dialog (presenting a number of templates)
 *
 * Usage:
 *  All dialog interaction is done via an object passed in and out as the
 *  first window argument: window.arguments[0]. On return
 *  window.arguments[0] has:
 *      .template       Full path to a template file selected by the user.
 *                      If the dialog was canceled then this is null.
 *      .addToToolbox   A boolean indicating if the identified template
 *                      should be added to the user's toolbox.
 *      .addToProject   NYI
 *      .project        NYI
 *      .filename       NYI
 *
 */
//TODO:
// - Allow multiple templates to be opened at once?


var log = getLoggingMgr().getLogger("templates");
//log.setLevel(LOG_DEBUG);

var gTemplateSvc = null;
var gCategoriesView = null;
var gTemplatesView = null;


//---- internal support routines


//---- interface routines for XUL

function OnLoad()
{
    try {
        var dialog = document.getElementById("dialog")
        var openButton = dialog.getButton("accept");
        openButton.setAttribute("label", "Open");
        openButton.setAttribute("accesskey", "O");
        openButton.setAttribute("disabled", "true");
        var openFolderButton = dialog.getButton("extra1");
        openFolderButton.setAttribute("label", "Open Template Folder");
        openFolderButton.setAttribute("accesskey", "F");
        openFolderButton.setAttribute("tooltiptext",
            "Open user template folder.  Place your own template files here.");
        var cancelButton = dialog.getButton("cancel");
        cancelButton.setAttribute("accesskey", "C");

        gTemplateSvc = Components.classes["@activestate.com/koTemplateService;1"].getService();
        gTemplatesView = Components.classes["@activestate.com/koTemplatesView;1"].createInstance();
        gCategoriesView = Components.classes["@activestate.com/koTemplateCategoriesView;1"].createInstance();
        gTemplateSvc.loadTemplates();
        gCategoriesView.initialize(gTemplateSvc, gTemplatesView);
        var categoriesTree = document.getElementById("categories");
        var boxObject = categoriesTree.treeBoxObject
                .QueryInterface(Components.interfaces.nsITreeBoxObject);
        boxObject.view = gCategoriesView;
        var templatesTree = document.getElementById("templates");
        boxObject = templatesTree.treeBoxObject
                .QueryInterface(Components.interfaces.nsITreeBoxObject);
        boxObject.view = gTemplatesView;
        
        var index = gCategoriesView.getDefaultCategoryIndex();
        gCategoriesView.selection.select(index);
    
        templatesTree.focus();
    } catch(ex) {
        log.exception(ex, "Error loading new file/templates dialog.");
    }
}

function OnUnload()
{
    gCategoriesView = null;
    gTemplatesView = null;
    gTemplateSvc = null;
}


function CategoriesOnSelectionChange()
{
    try {
        gCategoriesView.selectionChanged();
        // Select the last selected template in this category and ensure
        // it is visible.
        var index = gCategoriesView.getDefaultTemplateIndex();
        gTemplatesView.selection.select(index);
        var boxObject = document.getElementById("templates").treeBoxObject;
        boxObject.ensureRowIsVisible(index);
        _UpdateOpenButtonStatus();
    } catch(ex) {
        log.exception(ex, "Error changing categories selection.");
    }
}

function TemplatesOnSelectionChange()
{
    try {
        gCategoriesView.templateSelectionChanged();
        gTemplatesView.selectionChanged();
    } catch(ex) {
        log.exception(ex, "Error changing categories selection.");
    }
}


function _UpdateOpenButtonStatus()
{
    var openButton = document.getElementById("dialog").getButton("accept");
    if (gTemplatesView.rowCount && gTemplatesView.selection.count) {
        if (openButton.hasAttribute("disabled")) {
            openButton.removeAttribute("disabled");
        }
    } else {
        openButton.setAttribute("disabled", "true");
    }
}


function TemplatesOnClick(event) {
    try {
        // c.f. mozilla/mailnews/base/resources/content/threadPane.js
        var t = event.originalTarget;

        // double-click in the tree body and there is a selection
        if (event.detail == 2 && t.localName == "treechildren"
            && gTemplatesView.rowCount && gTemplatesView.selection.count) {
            // open the selected template
            if (Open()) {
                window.close();
            }
        }
    } catch(ex) {
        log.exception(ex, "Error handling templates tree click.");
    }
}


function Open()
{
    try {
        window.arguments[0].template = gTemplatesView.getSelectedTemplate();
        window.arguments[0].addToToolbox = document.getElementById("add-to-toolbox").checked;
        //XXX project stuff...
        
        //XXX add the template to the template MRU list
    
        gCategoriesView.saveSelections();
        return true;
    } catch(ex) {
        log.exception(ex, "Error returning selected template.");
    }
}

function OpenTemplateFolder()
{
    try {
        var os = Components.classes["@activestate.com/koOs;1"].getService();
        var dname = gTemplateSvc.getUserTemplatesDir();
    



        //XXX There has got to be a better way to do this on Linux.
        var currDir = os.getcwd();
        os.chdir(dname);
        os.system('xterm &');
        os.chdir(currDir);

    } catch(ex) {
        log.exception(ex, "Error opening templates folder.");
    }
}


function Cancel()
{
    try {
        window.arguments[0].template = null;
        return true;
    } catch(ex) {
        log.exception(ex, "Error canceling 'New File' dialog.");
    }
}


