from xpcom import nsError, ServerException, components
from xpcom.server.enumerator import SimpleEnumerator

import invocationutils

class koIInvocationInformation:
    _com_interfaces_ = components.interfaces.koIInvocationInformation
    _reg_clsid_ = "{f448f503-2b3a-4f63-abf4-99a842628e22}"
    _reg_contractid_ = "@activestate.com/InvocationInformation;1"
    def __init__(self):
        self.name = self.description = self.contractid = self.chrome_category = self.chrome_instance = None

class koIInvocationRegistryService:
    _com_interfaces_ = components.interfaces.koIInvocationRegistryService
    # If this object needs to be registered, the following 2 are also needed.
    _reg_clsid_ = "{1a67c86f-f330-4952-a415-94c0586bde01}"
    _reg_contractid_ = "@activestate.com/InvocationRegistryService;1"
    def __init__(self):
        self.invocation_to_info_map = None
        
    def _makeinfo(self,name,desc,id,cat,inst):
        info = koIInvocationInformation()
        info.name = name
        info.description = desc
        info.contractid = id
        info.chrome_category = cat
        info.chrome_instance = inst
        return info
        
    def _loadInvocations(self):
        assert self.invocation_to_info_map is None, "Already loaded!"
        self.invocation_to_info_map = {}

        info = self._makeinfo("Python","Python",
                            "@activestate.com/koInvocation;1?type=Python",
                            "chrome://jaguar/content/pref/invocations/categories/python.xul",
                            "chrome://jaguar/content/pref/invocations/instances/python.xul")
        self.invocation_to_info_map["Python"] = info

        info = self._makeinfo("Perl","Perl",
                            "@activestate.com/koInvocation;1?type=Perl",
                            "chrome://jaguar/content/pref/invocations/categories/perl.xul",
                            "chrome://jaguar/content/pref/invocations/instances/perl.xul")
        self.invocation_to_info_map["Perl"] = info

        info = self._makeinfo("PHP","PHP",
                            "@activestate.com/koInvocation;1?type=PHP",
                            "chrome://jaguar/content/pref/invocations/categories/php.xul",
                            "chrome://jaguar/content/pref/invocations/instances/php.xul")
        self.invocation_to_info_map["PHP"] = info

        info = self._makeinfo("XSLT","XSLT",
                            "@activestate.com/koInvocation;1?type=XSLT",
                            "chrome://jaguar/content/pref/invocations/categories/xslt.xul",
                            "chrome://jaguar/content/pref/invocations/instances/xslt.xul")
        self.invocation_to_info_map["XSLT"] = info

        info = self._makeinfo("Tcl","Tcl",
                            "@activestate.com/koInvocation;1?type=Tcl",
                            "chrome://jaguar/content/pref/invocations/categories/tcl.xul",
                            "chrome://jaguar/content/pref/invocations/instances/tcl.xul")
        self.invocation_to_info_map["Tcl"] = info

    def getInvocationInformation( self, name ):
        # Result: koIInvocationInformation
        # In: name: string
        if self.invocation_to_info_map is None:
            self._loadInvocations()
        try:
            return self.invocation_to_info_map[name]
        except KeyError:
            return None

    def enumerateInvocations( self ):
        # Result: nsISimpleEnumerator
        # return the list sorted by description (as that is how the UI is likely to want it)
        if self.invocation_to_info_map is None:
            self._loadInvocations()
        vals = self.invocation_to_info_map.values()
        vals.sort( lambda i1, i2: cmp(i1.description, i2.description) )
        return SimpleEnumerator(vals)

if __name__=='__main__':
    svc = components.classes["@activestate.com/InvocationRegistryService;1"].getService()
    enum = svc.enumerateInvocations()
    while enum.hasMoreElements():
        item = enum.getNext()
        print "Have invocation", item.name, item.description
