from xpcom import components, ServerException

from koLanguageServiceBase import *

class koPascalLanguage(KoLanguageBase):
    name = "Pascal"
    _reg_desc_ = "%s Language" % name
    _reg_contractid_ = "@activestate.com/koLanguage?language=%s;1" \
                       % (name)
    _reg_clsid_ = "{8AE35E4C-0EC9-49f2-A534-8FEAB91D261D}"

    defaultExtension = ".pas"
    commentDelimiterInfo = {
        # See the following for info on Pascal comments:
        #  http://www.math.uni-hamburg.de/it/software/fpk/ref/node7.html
        #XXX The line Pascal comment is a Delphi only thing, so I am leaving
        #    it out for now.
        #"line": [ "//" ],
        "block": [ ("{",  "}"),
                   ("(*", "*)") ],
    }
    _comment_styles = [sci_constants.SCE_P_COMMENTLINE,
                       sci_constants.SCE_P_COMMENTBLOCK,]
    supportsSmartIndent = "brace"
    
    def get_lexer(self):
        if self._lexer is None:
            self._lexer = KoLexerLanguageService()
            self._lexer.setLexer(components.interfaces.ISciMoz.SCLEX_PASCAL)
            self._lexer.setKeywords(0, self._keywords)
            self._lexer.supportsFolding = 1
        return self._lexer

    def get_commenter(self):
        if self._commenter is None:
            self._commenter = KoCommenterLanguageService(
                self.commentDelimiterInfo)
        return self._commenter

    _keywords = ["program", "const", "type", "var", "begin", "end",
                 "array", "set", "record", "string", "if", "then", "else", "while",
                 "for", "to", "downto", "do", "with", "repeat", "until", "case",
                 "of", "goto", "exit", "label", "procedure", "function", "nil",
                 "file", "and", "or", "not", "xor", "div", "mod", "unit", "uses",
                 "implementation", "interface", "external", "asm", "inline",
                 "object", "constructor", "destructor", "virtual", "far",
                 "assembler", "near", "inherited"] 

