# Copyright (c) 2000-2001 ActiveState Corporation.
# See the file LICENSE.txt for licensing information.

from xpcom import components, _xpcom
import threading
import socket
import os
import string
import logging

PROXY_SYNC = 1
PROXY_ASYNC = 2
PROXY_ALWAYS = 4

#    var clz = Components.classes["@activestate.com/debugger;1"];
#    dbgd.dbg = clz.getService(Components.interfaces.koIDebugger);
#    clz = Components.classes["@mozilla.org/network/protocol;1?name=kodebugger"]
#    var handler = clz.getService(Components.interfaces.koIDebuggerProtocolHandler);

log = logging.getLogger("koRemoteDebuggerListener")


class KoRemoteDebugConnectionListener:
    _com_interfaces_= [components.interfaces.koIDebugConnectionListener]
    _reg_clsid_ = "{9B7403B0-10EF-429e-9415-23F6A2CB4CC1}"
    _reg_contractid_ = "@activestate.com/RemoteDebugConnectionListener;1"
    
    def __init__(self):
        self.perlListener = None
        self._kothread = None
        self._perlthread = None
        
    def registerHandler(self,handler):
        self._handler = _xpcom.getProxyForObject(0, 
	       components.interfaces.koIDebuggerConnectionEventHandler,
	       handler, PROXY_ASYNC | PROXY_ALWAYS)
        self._cancel = 0

    def listen(self,port):
        raise "Not implemented: use listenNewThread"

    def _init_perl_listener(self):
        # Setup the environment for the Perl debugger and create a Perl
        # debugger component.
        koDirs = components.classes["@activestate.com/koDirs;1"].\
            getService(components.interfaces.koIDirs)

        # Override the PDK setting
        oldPerl5DB = os.environ.get("PERL5DB", "")
        os.environ["PERL5DB"]="BEGIN { require 'perl5db.pl' }"

        # prepend Komodo's Perl utils dir to PERL5LIB
        oldPerl5LibDirs = os.environ.get("PERL5LIB", "")
        perl5LibDirs = os.environ.get("PERL5LIB", "").split(os.pathsep)
        perlUtilsDir = koDirs.perlUtilsDir
        for d in perl5LibDirs:
            if os.path.normpath(os.path.abspath(perlUtilsDir)) ==\
               os.path.normpath(os.path.abspath(d)):
                break
        else:
            perl5LibDirs.insert(0, perlUtilsDir)
        os.environ["PERL5LIB"] = os.pathsep.join(perl5LibDirs)

        self.perlListener = components.classes["@activestate.com/koPerlDebuggerListener;1"] \
              .createInstance (components.interfaces.koIPerlDbgListener);
        
        os.environ["PERL5LIB"] = oldPerl5LibDirs
        os.environ["PERL5DB"] = oldPerl5DB
    
    def listenNewThread(self,port,perlPort):
        self.port = port
        self.perlPort = perlPort
        #log.debug("Listening on %s" % self.port)
        
        if not (self._kothread and self._kothread.isAlive()):
            self._cancel = 0
            self._kothread = threading.Thread(target = self._listen)
            self._kothread.start()
            
        if perlPort and \
           not (self._perlthread and self._perlthread.isAlive()):
            self._cancel = 0
            # start a perl debugger thread
            self._init_perl_listener()
            self._perlthread = threading.Thread(target = self._perlListen)
            self._perlthread.start()

    def cancelListen(self):
        # log.debug("In cancelListen, about to connect to %d" % (self.port,))
        if not self._cancel:
            self._cancelRemoteListen(self.port)
            self._cancelRemoteListen(self.perlPort)

    def _cancelRemoteListen(self, port):
        if port:
            self._cancel = 1
            try:
                s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                s.connect(('127.0.0.1', port))
                s.send('0:0:0')
                s.close()
            except socket.error, details:
                log.error("The debugger listener could not cancel the listener "+str(port)+":" + str(details))
                pass
                
    def _perlListen(self):
        res = self.perlListener.Init(str(self.perlPort) + ':0:0:0')
        
        if self._cancel:
            self.perlListener.Term()
            self.perlListener = None
        else:
            # cancel the other listener
            if self._kothread and self._kothread.isAlive():
                self._cancelRemoteListen(self.port)
            # get the info we need!
            host = self.perlListener.DoCommand("PeerHost");
            self._handler.on_connect("Perl", host, self.perlPort, "")
    
    def _listen(self):
        #XXX: Use udp, because this is all we need.
        tm = None
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            # try to re-use a server port if possible
            s.setsockopt(
                socket.SOL_SOCKET, socket.SO_REUSEADDR,
                s.getsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR) | 1)
            s.bind(('', self.port))
            s.listen(1)
            (client, addr) = s.accept()
            # client is a new socket, close s
            tm = client.recv(1024)
            client.close()
            s.close()
        except socket.error, details:
            log.error("The debugger listener could not start listening:" + str(details))
            try:
                self._handler.on_error(details[1])
            except COMException:
                    pass

        if not self._cancel:
            # log.debug("Got a connection from (%s)" %(addr))
            # need to set a timeout here to handle perl without a big slowdown
    
            if tm:
                # later change this to recieve proper HTML headers
                rargs = tm.split(":")
                if not rargs or len(rargs) != 3:
                    log.error("Invalid handshake received: %s"%(tm))
                    self._handler.on_quit()
                    return
                
                language = rargs[0];
                port = string.atoi(rargs[1])
                cookie = rargs[2]
                self._handler.on_connect(language, addr[0], port, cookie)

        if self._perlthread and self._perlthread.isAlive():
            self._cancelRemoteListen(self.perlPort)

