#!/usr/bin/env python
# Copyright (c) 2000-2001 ActiveState Corporation.
# See the file LICENSE.txt for licensing information.

# Launch a URL in the user's "default" browser.
#   We used to use "webbrowser.py" here but it does netscape magic and
#   allows for command line browsers, both of which can cause Komodo to hang.

from xpcom import components
from xpcom import components, nsError, ServerException
import process, koprocessutils
import sys, os
import logging
import which

log = logging.getLogger('koWebbrowser')

class KoWebbrowser:
    _com_interfaces_ = [components.interfaces.koIWebbrowser]
    _reg_clsid_ = "{7B361CD6-4426-4d23-86D6-B6F8E0FCDA20}"
    _reg_contractid_ = "@activestate.com/koWebbrowser;1"
    _reg_desc_ = "Komodo Default Webbrowser Service"

    def open_new(self, url):
        """Open the given URL in the user's default browser.
        Return true iff successful.
        """
        prefs = components.classes["@activestate.com/koPrefService;1"].\
                getService(components.interfaces.koIPrefService).prefs
        if sys.platform.startswith("win") and prefs.getStringPref('browser') == '':
            try:
                os.startfile(url)
                return 1
            except WindowsError, e:
                # I don't know who is at fault here but if Netscape 6 (and
                # presumable, therefore, some versions of Mozilla) is set as
                # the default then (1) the browser successfully loads the
                # file *and* (2) this is raised:
                #    Traceback (most recent call last):
                #      File "C:\mozilla_source\moz.09apr.trentm.winnt.release.komodo5\mozilla\dist\WIN32_O.OBJ\bin\components\koWebbrowser.py", line 27, in open_new
                #        os.startfile(url)
                #    WindowsError: [Errno 2] The system cannot find the file specified: 'http://www.ActiveState.com/ASPN/Downloads/Komodo/NeedLicense?os=Windows2000v.5.0.2195&build=19520&version=1.1.0-devel'
                #
                if e.errno == 2:
                    return 1
                else:
                    return 0
            except:
                return 0
        browser = prefs.getStringPref('browser')
        return open_new_browser(url, browser)

    def open_new_browser(self, url, browser):
        # process will quote args with spaces, make sure it's
        # not already quoted
        if browser[0] == '"':
            browser = browser[1:-1]
        try:
            env = koprocessutils.getUserEnv()
        except ServerException, ex:
            log.error(str(ex))
            return 1
        try:
            process.Process([browser, url], env=env)
            return 1
        except process.ProcessError, ex:
            log.error(str(ex))
            return 0

    def _SameFile(self, fname1, fname2):
        return ( os.path.normpath(os.path.normcase(fname1)) ==\
            os.path.normpath(os.path.normcase(fname2)) )

    def get_possible_browsers(self):
        # Only search the PATH as it was when Komodo started, otherwise
        # Komodo's internal mozilla.exe might be listed as a possible browser.
        #   http://bugs.activestate.com/Komodo/show_bug.cgi?id=26373
        path = koprocessutils.getUserEnv()["PATH"]
        path = path.split(os.pathsep)
        if sys.platform.startswith('win'):
            matches = (
                which.whichall("iexplore", exts=['.exe'], path=path) +
                which.whichall("mozilla", exts=['.exe'], path=path) +
                which.whichall("MozillaFirebird", exts=['.exe'], path=path) +
                which.whichall("opera", exts=['.exe'], path=path) +
                which.whichall("netscape", exts=['.exe'], path=path)
            )
        else:
            matches = (
                which.whichall("mozilla", path=path) +
                which.whichall("MozillaFirebird", path=path) +
                which.whichall("konqueror", path=path) +
                which.whichall("opera", path=path) +
                which.whichall("netscape", path=path) +
                which.whichall("netscape-communicator", path=path) +
                which.whichall("kfm", path=path)
            )
        # check for duplicates
        matchesWithoutDups = []
        for i in range(len(matches)):
            for j in range(i+1, len(matches)):
                if self._SameFile(matches[i], matches[j]):
                    break
            else:
                matchesWithoutDups.append(matches[i])
        return matchesWithoutDups
